@@ -33,9 +33,10 @@
 #include "rs232.c"
 
 #define HEX(c) (int)(((c) < 'A') ? ((c) - '0') : ((c) - 'A') + 10)
+#define MAX_STEP_FREQ 1000
 
 #ifndef GAMMAJ2000
-///Right ascension of the meridian at J2000 zero at Greenwich
+///Right ascension of the sun at J2000 zero from Greenwich
 #define GAMMAJ2000 18.6971378528
 #endif
 #ifndef SIDEREAL_DAY
@@ -72,6 +73,7 @@ typedef struct {
     double maxperiod[num_axes];
     double minperiod[num_axes];
     double speed_limit[num_axes];
+    double max_step_frequency[num_axes];
     double acceleration[num_axes];
     double crown[num_axes];
     double steps[num_axes];
@@ -99,6 +101,7 @@ typedef struct {
     int will_flip;
     int tracking_mode;
     int threads_running;
+    int baud_rate;
     pthread_t tracking_thread;
 } gt1_info;
 
@@ -112,7 +115,7 @@ static int dispatch_command(SkywatcherCommand cmd, int axis, int command_arg);
 static unsigned int ahp_gt_current_device = 0;
 static unsigned int ahp_gt_connected = 0;
 static unsigned int ahp_gt_detected[128] = { 0 };
-static gt1_info devices[128];
+static gt1_info devices[128] = { 0 };
 static int sockfd;
 
 static double get_timestamp()
@@ -142,12 +145,12 @@ static double time_to_J2000time(time_t tp)
 static double J2000time_to_lst(double secs_since_J2000, double Long)
 {
     Long *= SIDEREAL_24 / 360.0;
-    return fmod(SIDEREAL_24 * secs_since_J2000 / SIDEREAL_DAY + Long + GAMMAJ2000, SIDEREAL_24);
+    return fmod(SIDEREAL_24 * secs_since_J2000 / SIDEREAL_DAY + Long - GAMMAJ2000, SIDEREAL_24);
 }
 
 static double get_lst()
 {
-    double lat, lon, el;
+    double lat = 0.0, lon = 0.0, el = 0.0;
     ahp_gt_get_location(&lat, &lon, &el);
     double lst = fmod(J2000time_to_lst(time_to_J2000time(ahp_gt_get_time()), lon), 24.0);
     return lst;
@@ -171,9 +174,18 @@ static double range_24(double ra)
     return ra;
 }
 
+static double range_180(double deg)
+{
+    if (deg >= 180.0)
+        return (360.0 - deg);
+    if (deg < -180.0)
+        return (deg + 360.0);
+    return deg;
+}
+
 static double range_360(double deg)
 {
-    if ((deg >= 360.0))
+    if (deg >= 360.0)
         return (deg - 360.0);
     if (deg < 0.0)
         return (deg + 360.0);
@@ -191,24 +203,49 @@ double range_dec(double dec)
     return dec;
 }
 
+static double get_flipped_ha(double *ha, double *dec)
+{
+    int flipped = 0;
+    double lat = 0.0, lon = 0.0, el = 0.0;
+    ahp_gt_get_location(&lat, &lon, &el);
+    *ha = range_24(*ha);
+    if (lat >= 0.0) {
+        if(*dec > 0.0) {
+            *ha = *ha-12.0;
+            flipped = 1;
+        }
+    }
+    if (lat < 0.0) {
+        if(*dec < 0.0) {
+            *ha = *ha-12.0;
+            flipped = 1;
+        }
+    }
+    *ha = range_24(*ha);
+    return flipped;
+}
+
 static double calc_flipped_ha(double *ha, double *dec)
 {
     int flipped = 0;
-    double lat, lon, el;
+    double lat = 0.0, lon = 0.0, el = 0.0;
     ahp_gt_get_location(&lat, &lon, &el);
-    *ha = range_ha(*ha);
-    *dec -= 90.0;
-    if (lat >= 0.0 && *ha < 0.0) {
-        flipped = 1;
-            *ha = *ha+12.0;
-        *dec = -*dec;
+    if (lat >= 0.0) {
+        if(*ha > 0.0) {
+            *ha = *ha-6.0;
+        } else {
+            *ha = *ha+6.0;
+            *dec = -*dec;
+        }
     }
-    if (lat < 0.0 && *ha >= 0.0) {
-        flipped = 1;
-            *ha = *ha-12.0;
-        *dec = -*dec;
+    if (lat < 0.0) {
+        if(*ha < 0.0) {
+            *ha = *ha-6.0;
+        } else {
+            *ha = *ha+6.0;
+            *dec = -*dec;
+        }
     }
-    *dec += 90.0;
     return flipped;
 }
 
@@ -218,6 +255,23 @@ static double get_local_hour_angle(double Ra)
     return range_ha(ha);
 }
 
+void radec2rad(double ra, double dec, double *alpha, double *delta, int *flipped)
+{
+    dec = range_dec(dec);
+    ra = range_24(ra);
+    double ha = get_local_hour_angle(ra);
+    dec -= 90.0;
+    if((ahp_gt_get_mount_flags() & isForkMount) == 0) {
+        *flipped = calc_flipped_ha(&ha, &dec);
+    } else {
+        ha -= 6.0;
+        *flipped = 0;
+    }
+    ha = range_ha(ha);
+    *alpha = ha * M_PI / 12.0;
+    *delta = dec * M_PI / 180.0;
+}
+
 double ahp_gt_tracking_sine(double Alt, double Az, double Lat)
 {
     Alt *= M_PI / 180.0;
@@ -264,9 +318,9 @@ void ahp_gt_get_ra_dec_coordinates(double Alt, double Az, double *Ra, double *De
     dec = asin(cos(Az) * (cos(Alt) * cos(lat)) + sin(Alt) * sin(lat));
     ha = acos((sin(Alt) - sin(dec) * sin(lat)) / (cos(dec) * cos(lat)));
     dec *= 180.0 / M_PI;
-    ha *= 24.0 / M_PI;
     if (sin(ha) > 0.0)
-        ha = 24.0 - ha;
+        ha = M_PI * 2.0 - ha;
+    ha *= 24.0 / M_PI;
     ra = range_24(get_local_hour_angle(ha));
     *Ra = ra;
     *Dec = dec;
@@ -274,8 +328,8 @@ void ahp_gt_get_ra_dec_coordinates(double Alt, double Az, double *Ra, double *De
 
 double ahp_gt_get_ha()
 {
-    double lat, lon, el;
-    double ha = ahp_gt_get_position(0, NULL) * 12.0 / M_PI;
+    double lat = 0.0, lon = 0.0, el = 0.0;
+    double ha = ahp_gt_get_position(0, NULL) * 12.0 / M_PI + 6.0;
     ahp_gt_get_location(&lat, &lon, &el);
     if (lat >= 0.0)
         ha = range_ha(ha);
@@ -286,35 +340,28 @@ double ahp_gt_get_ha()
 
 double ahp_gt_get_ra()
 {
-    double lat, lon, el;
-    ahp_gt_get_location(&lat, &lon, &el);
-    double ra = get_lst() - range_24(ahp_gt_get_ha());
-    if(devices[ahp_gt_get_current_device()].flipped) {
-        if (lat >= 0.0) {
-            ra = get_lst() - range_24(ahp_gt_get_ha()) - 12.0;
-        } else {
-            ra = get_lst() - range_24(ahp_gt_get_ha()) + 12.0;
-        }
+    double ha = ahp_gt_get_ha();
+    double dec = ahp_gt_get_position(1, NULL) * 180.0 / M_PI;
+    if((ahp_gt_get_mount_flags() & isForkMount) == 0) {
+        devices[ahp_gt_get_current_device()].flipped = get_flipped_ha(&ha, &dec);
+    } else {
+        devices[ahp_gt_get_current_device()].flipped = 0;
     }
-    return range_24(ra);
+    return range_24(get_lst() - ha);
 }
 
 double ahp_gt_get_dec()
 {
+    double ha = ahp_gt_get_ha();
     double dec = ahp_gt_get_position(1, NULL) * 180.0 / M_PI;
-    double lat, lon, el;
-    ahp_gt_get_location(&lat, &lon, &el);
-    if(lat >= 0.0) {
-        if(dec > 90.0 && dec < 270.0)
-            devices[ahp_gt_get_current_device()].flipped = 1;
-        else
-            devices[ahp_gt_get_current_device()].flipped = 0;
+    if((ahp_gt_get_mount_flags() & isForkMount) == 0) {
+        double delta = dec;
+        devices[ahp_gt_get_current_device()].flipped = get_flipped_ha(&ha, &delta);
     } else {
-        if(dec < 90.0 || dec >= 270.0)
-            devices[ahp_gt_get_current_device()].flipped = 1;
-        else
-            devices[ahp_gt_get_current_device()].flipped = 0;
+        devices[ahp_gt_get_current_device()].flipped = 0;
     }
+    dec += 90.0;
+    dec = range_dec(dec);
     return dec;
 }
 
@@ -478,13 +525,13 @@ static int synscan_poll()
                 ahp_gt_stop_motion(1, 0);
                 break;
             case GetTrackingMode:
-                sprintf(msg, "%c#", ahp_gt_is_axis_moving(0) * 2);
+                sprintf(msg, "%c#", ahp_gt_get_tracking_mode());
                 write(devices[ahp_gt_get_current_device()].connfd, msg, 2);
                 break;
             case SetTrackingMode:
                 if(readcmd(cmd, 1) < 0)
                     goto err_end;
-                devices[ahp_gt_get_current_device()].tracking_mode = cmd[0];
+                ahp_gt_get_tracking_mode(cmd[0]);
                 sprintf(msg, "#");
                 write(devices[ahp_gt_get_current_device()].connfd, msg, 1);
                 break;
@@ -663,36 +710,42 @@ static void long2Revu24str(unsigned int n, char *str)
 
 static int read_eqmod()
 {
+    char * reply;
     int err_code = 0, nbytes_read = 0;
-    int max_err = 100;
+    int max_err = 30;
     // Clear string
-    response[0] = '\0';
+    memset(response, '\0', 32);
     unsigned char c = 0;
     while(c != '\r' && err_code < max_err) {
         if(1 == ahp_serial_RecvBuf(&c, 1) && c != 0) {
             response[nbytes_read++] = c;
         } else {
             err_code++;
-            usleep(100);
+            usleep(1000);
         }
     }
     if (err_code == max_err)
     {
         return 0;
     }
+    if(!strncmp(command, response, strlen(command))) {
+        reply = &response[strlen(command)];
+        nbytes_read -= strlen(command);
+    } else reply = response;
+
     // Remove CR
-    response[nbytes_read - 1] = '\0';
+    reply[nbytes_read - 1] = '\0';
 
-    pwarn("%s\n", response);
+    pwarn("%s\n", reply);
 
-    switch (response[0])
+    switch (reply[0])
     {
         case '=':
             if(nbytes_read > 2) {
                 if(nbytes_read > 5)
-                    return Revu24str2long(response+1);
+                    return Revu24str2long(reply+1);
                 else
-                    return Highstr2long(response+1);
+                    return Highstr2long(reply+1);
             }
             break;
         case '!':
@@ -707,10 +760,9 @@ static int read_eqmod()
 static int dispatch_command(SkywatcherCommand cmd, int axis, int arg)
 {
     int ret = -1;
-    int maxtries = 20;
-    int nbytes_read = 0;
+    int maxtries = 15;
     int i;
-    unsigned char c = 0;
+    int c;
     memset(response, '0', 32);
     if(!mutexes_initialized) {
         pthread_mutexattr_init(&mutex_attr);
@@ -720,36 +772,31 @@ static int dispatch_command(SkywatcherCommand cmd, int axis, int arg)
     }
     while(pthread_mutex_trylock(&mutex))
         usleep(100);
-    for(i = 0; i < maxtries; i++)
-    {
-        // Clear string
-        command[0] = '\0';
-        char command_arg[28];
-        int n;
-        if (arg < 0) {
-            snprintf(command, 32, ":%c%c\r", cmd, (char)(axis+'1'));
-            n = 4;
-        } else {
-            arg = (int)fmin((double)0xffffff, arg);
-            long2Revu24str((unsigned int)arg, command_arg);
-            snprintf(command, 32, ":%c%c%s\r", (char)cmd, (char)(axis+'1'), command_arg);
-            n = 10;
-        }
-        pgarb("%s\n", command);
-
-        ahp_serial_flushRXTX();
-        if ((ahp_serial_SendBuf((unsigned char*)command, n)) < n)
-        {
-            if (i == maxtries-1)
-            {
-                goto ret_err;
-            }
-        }
-        command[n-1] = '\0';
-
-        ret = read_eqmod();
-        break;
+    i = maxtries;
+retry:
+    command[0] = '\0';
+    char command_arg[28];
+    int n;
+    if (arg < 0) {
+        snprintf(command, 32, ":%c%c\r", cmd, (char)(axis+'1'));
+        n = 4;
+    } else {
+        arg = (int)fmin((double)0xffffff, arg);
+        long2Revu24str((unsigned int)arg, command_arg);
+        snprintf(command, 32, ":%c%c%s\r", (char)cmd, (char)(axis+'1'), command_arg);
+        n = 10;
     }
+    pgarb("%s\n", command);
+
+    ahp_serial_flushRXTX();
+    for(c = 0; c < n; c++) {
+        if ((ahp_serial_SendByte((unsigned char)command[c])) < 0)
+            goto retry;
+    }
+    if (--i == 0)
+        goto ret_err;
+
+    ret = read_eqmod();
     pthread_mutex_unlock(&mutex);
     return ret;
 ret_err:
@@ -779,17 +826,29 @@ static void optimize_values(int axis)
         usteps --;
     }
     devices[ahp_gt_get_current_device()].multiplier [axis] = 1;
-    if(devices[ahp_gt_get_current_device()].stepping_mode[axis] != HalfStep)
+    switch(devices[ahp_gt_get_current_device()].stepping_mode[axis]) {
+    case HalfStep:
+        devices[ahp_gt_get_current_device()].speed_limit [axis] = MAX_STEP_FREQ*256;
+        break;
+    case Microstep:
         devices[ahp_gt_get_current_device()].multiplier [axis] += (int)usteps;
+        devices[ahp_gt_get_current_device()].speed_limit [axis] = MAX_STEP_FREQ/4;
+        break;
+    case Mixed:
+        devices[ahp_gt_get_current_device()].multiplier [axis] += (int)usteps;
+        devices[ahp_gt_get_current_device()].speed_limit [axis] = MAX_STEP_FREQ;
+        break;
+    default:
+        break;
+    }
     devices[ahp_gt_get_current_device()].one_second[axis] = (1500000.0);
     devices[ahp_gt_get_current_device()].wormsteps [axis] *= (double)devices[ahp_gt_get_current_device()].multiplier [axis] / (double)devices[ahp_gt_get_current_device()].divider [axis];
     devices[ahp_gt_get_current_device()].totalsteps [axis] = (int)(devices[ahp_gt_get_current_device()].crown [axis] * devices[ahp_gt_get_current_device()].wormsteps [axis]);
 
     double sidereal_period = SIDEREAL_DAY * devices[ahp_gt_get_current_device()].multiplier[axis] * devices[ahp_gt_get_current_device()].wormsteps[axis] / devices[ahp_gt_get_current_device()].totalsteps[axis];
     devices[ahp_gt_get_current_device()].maxperiod [axis] = (int)sidereal_period;
-    devices[ahp_gt_get_current_device()].speed_limit [axis] = (int)(2000);
+    devices[ahp_gt_get_current_device()].speed_limit [axis] *= (SIDEREAL_DAY * (double)devices[ahp_gt_get_current_device()].multiplier[axis] / devices[ahp_gt_get_current_device()].totalsteps [axis]);
     devices[ahp_gt_get_current_device()].minperiod [axis] = 1;
-    devices[ahp_gt_get_current_device()].maxspeed [axis] = fmin(devices[ahp_gt_get_current_device()].speed_limit [axis], devices[ahp_gt_get_current_device()].maxspeed [axis]);
     devices[ahp_gt_get_current_device()].maxspeed_value [axis] = (int)fmax(devices[ahp_gt_get_current_device()].minperiod [axis], (devices[ahp_gt_get_current_device()].maxperiod [axis] / devices[ahp_gt_get_current_device()].maxspeed [axis]));
     devices[ahp_gt_get_current_device()].guide [axis] = (int)(SIDEREAL_DAY * baseclock / devices[ahp_gt_get_current_device()].totalsteps [axis]);
 
@@ -965,7 +1024,7 @@ void ahp_gt_read_values(int axis)
     devices[ahp_gt_get_current_device()].accel_steps [axis] = ((tmp >> 18) & 0x3f);
     devices[ahp_gt_get_current_device()].accel_increment [axis] =  (tmp >> 10) & 0xff;
     devices[ahp_gt_get_current_device()].multiplier [axis] = (tmp >> 3) & 0x7f;
-    devices[ahp_gt_get_current_device()].direction_invert [axis] = (tmp >> 2) & 0x1;
+    devices[ahp_gt_get_current_device()].direction_invert [axis] = tmp & 0x1;
     devices[ahp_gt_get_current_device()].stepping_conf [axis] = (tmp & 0x06)>>1;
     devices[ahp_gt_get_current_device()].features [axis] = dispatch_command(GetVars, offset + 6, -1);
     devices[ahp_gt_get_current_device()].gt1feature[axis] = dispatch_command(GetVars, offset + 7, -1) & 0x7;
@@ -1067,9 +1126,10 @@ int ahp_gt_connect(const char* port)
     if(ahp_gt_is_connected())
         return 0;
     if(!ahp_serial_OpenComport(port)) {
-        int rate = 9600;
+        devices[ahp_gt_get_current_device()].baud_rate = 9600;
+        int highspeed = 0;
 retry:
-        if(!ahp_serial_SetupPort(rate, "8N1", 0)) {
+        if(!ahp_serial_SetupPort(devices[ahp_gt_get_current_device()].baud_rate, "8N1", 0)) {
             ahp_gt_connected = 1;
             memset(devices, 0, sizeof(gt1_info)*128);
             memset(ahp_gt_detected, 0, sizeof(unsigned int)*128);
@@ -1079,8 +1139,9 @@ retry:
                     pgarb("MC Version: %02X\n", devices[ahp_gt_get_current_device()].version);
                     return 0;
                 }
-            } else if(rate == 9600) {
-                rate = 115200;
+            } else if(!highspeed) {
+                devices[ahp_gt_get_current_device()].baud_rate = 115200;
+                highspeed = 1;
                 goto retry;
             }
         }
@@ -1096,8 +1157,7 @@ void ahp_gt_disconnect()
         for(addr = 0; addr < 128; addr++) {
             if(ahp_gt_detected[addr]) {
                 ahp_gt_select_device(addr);
-                devices[ahp_gt_get_current_device()].threads_running = 0;
-                pthread_join(devices[ahp_gt_get_current_device()].tracking_thread, NULL);
+                ahp_gt_stop_tracking_thread();
             }
         }
         ahp_serial_CloseComport();
@@ -1278,8 +1338,14 @@ double ahp_gt_get_max_speed(int axis)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return 0.0;
-    double sidereal_speed = SIDEREAL_DAY * ahp_gt_get_multiplier(axis) * ahp_gt_get_wormsteps(axis) / ahp_gt_get_totalsteps(axis);
-    return sidereal_speed / devices[ahp_gt_get_current_device()].maxspeed_value[axis];
+    return devices[ahp_gt_get_current_device()].maxspeed [axis];
+}
+
+double ahp_gt_get_max_step_frequency(int axis)
+{
+    if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
+        return 0.0;
+    return devices[ahp_gt_get_current_device()].max_step_frequency[axis];
 }
 
 double ahp_gt_get_speed_limit(int axis)
@@ -1426,6 +1492,14 @@ void ahp_gt_set_max_speed(int axis, double value)
     optimize_values(axis);
 }
 
+void ahp_gt_set_max_step_frequency(int axis, double value)
+{
+    if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
+        return;
+    devices[ahp_gt_get_current_device()].max_step_frequency[axis] = value;
+    optimize_values(axis);
+}
+
 void ahp_gt_set_divider(int axis, int value)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
@@ -1464,6 +1538,7 @@ int ahp_gt_detect_device() {
         return -1;
     int a = 0;
     ahp_gt_detected[ahp_gt_get_current_device()] = 0;
+    devices[ahp_gt_get_current_device()].baud_rate = 9600;
     dispatch_command(SetAddress, 0, ahp_gt_current_device);
     if(ahp_gt_get_mc_version() > 0) {
         for (a = 0; a < num_axes; a++) {
@@ -1487,6 +1562,7 @@ void ahp_gt_set_address(int address)
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
     devices[ahp_gt_get_current_device()].address_value = address;
+    optimize_values(0);
 }
 
 int ahp_gt_get_address()
@@ -1526,14 +1602,14 @@ void ahp_gt_set_position(int axis, double value)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
-    dispatch_command(SetAxisPositionCmd, axis, (int)(value*devices[ahp_gt_get_current_device()].totalsteps[axis]/M_PI/2.0)+0x800000);
+    dispatch_command(SetAxisPositionCmd, axis, (int)((value-M_PI_2)*devices[ahp_gt_get_current_device()].totalsteps[axis]/M_PI/2.0)+0x800000);
 }
 
 double ahp_gt_get_position(int axis, double *timestamp)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return 0;
-    int steps = dispatch_command(GetAxisPosition, axis, -1)-8388608;
+    int steps = dispatch_command(GetAxisPosition, axis, -1)-0x800000;
     if(timestamp != NULL)
         *timestamp = get_timestamp() - 0.008333333;
     return (double)steps*M_PI*2.0/(double)devices[ahp_gt_get_current_device()].totalsteps[axis];
@@ -1555,10 +1631,26 @@ double ahp_gt_get_time()
     return devices[ahp_gt_get_current_device()].time_offset + t;
 }
 
+void ahp_gt_set_time_offset(double offset)
+{
+    if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
+        return;
+    devices[ahp_gt_get_current_device()].time_offset = offset;
+}
+
+double ahp_gt_get_time_offset()
+{
+    if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
+        return 0.0;
+    return devices[ahp_gt_get_current_device()].time_offset;
+}
+
 void ahp_gt_set_location(double latitude, double longitude, double elevation)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
+    longitude = range_360(longitude);
+    latitude = range_dec(latitude);
     devices[ahp_gt_get_current_device()].lat = latitude;
     devices[ahp_gt_get_current_device()].lon = longitude;
     devices[ahp_gt_get_current_device()].el = elevation;
@@ -1584,6 +1676,10 @@ void ahp_gt_goto_altaz(double alt, double az)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
+    if(ahp_gt_is_axis_moving(0))
+        return;
+    if(ahp_gt_is_axis_moving(1))
+        return;
     double ra, dec;
     ahp_gt_get_ra_dec_coordinates(alt, az, &ra, &dec);
     ahp_gt_goto_radec(ra, dec);
@@ -1593,43 +1689,37 @@ void ahp_gt_goto_radec(double ra, double dec)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
-    double ha = get_local_hour_angle(ra);
-    if((ahp_gt_get_mount_flags() & isForkMount) == 0) {
-        devices[ahp_gt_get_current_device()].will_flip = calc_flipped_ha(&ha, &dec);
-    } else {
-        devices[ahp_gt_get_current_device()].will_flip = 0;
-    }
-    dec *= M_PI / 180.0;
-    ha *= M_PI / 12.0;
-    ahp_gt_goto_absolute(0, ha, 800.0);
-    ahp_gt_goto_absolute(1, dec, 800.0);
+    if(ahp_gt_is_axis_moving(0))
+        return;
+    if(ahp_gt_is_axis_moving(1))
+        return;
+    double alpha, delta;
+    radec2rad(ra, dec, &alpha, &delta, &devices[ahp_gt_get_current_device()].will_flip);
+    ahp_gt_goto_absolute(0, alpha, 800.0);
+    ahp_gt_goto_absolute(1, delta, 800.0);
 }
 
 void ahp_gt_sync_radec(double ra, double dec)
 {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
-    double ha = get_local_hour_angle(ra);
-    if((ahp_gt_get_mount_flags() & isForkMount) == 0) {
-        devices[ahp_gt_get_current_device()].will_flip = calc_flipped_ha(&ha, &dec);
-    } else {
-        devices[ahp_gt_get_current_device()].will_flip = 0;
-    }
-    dec *= M_PI / 180.0;
-    ha *= M_PI / 12.0;
-    ahp_gt_set_position(0, ha);
-    ahp_gt_set_position(1, dec);
+    double alpha, delta;
+    radec2rad(ra, dec, &alpha, &delta, &devices[ahp_gt_get_current_device()].flipped);
+    ahp_gt_set_position(0, alpha);
+    ahp_gt_set_position(1, delta);
 }
 
 void ahp_gt_goto_absolute(int axis, double target, double speed) {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
+    if(ahp_gt_is_axis_moving(axis))
+        return;
     double position = ahp_gt_get_position(axis, NULL);
     speed = fabs(speed);
     speed *= (target-position < 0 ? -1 : 1);
     double max = devices[ahp_gt_get_current_device()].totalsteps[axis];
-    target /= M_PI*2;
     target *= max;
+    target /= M_PI*2;
     target += (double)0x800000;
     double maxperiod = SIDEREAL_DAY * devices[ahp_gt_get_current_device()].wormsteps[axis] / devices[ahp_gt_get_current_device()].totalsteps[axis];
     int period = maxperiod * devices[ahp_gt_get_current_device()].multiplier[axis];
@@ -1653,6 +1743,8 @@ void ahp_gt_goto_absolute(int axis, double target, double speed) {
 void ahp_gt_goto_relative(int axis, double increment, double speed) {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
+    if(ahp_gt_is_axis_moving(axis))
+        return;
     speed = fabs(speed);
     speed *= (increment < 0 ? -1 : 1);
     double max = devices[ahp_gt_get_current_device()].totalsteps[axis];
@@ -1679,6 +1771,8 @@ void ahp_gt_goto_relative(int axis, double increment, double speed) {
 void ahp_gt_start_motion(int axis, double speed) {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
+    if(ahp_gt_is_axis_moving(axis))
+        return;
     double period = SIDEREAL_DAY * devices[ahp_gt_get_current_device()].multiplier[axis] * devices[ahp_gt_get_current_device()].wormsteps[axis] / devices[ahp_gt_get_current_device()].totalsteps[axis];
     SkywatcherMotionMode mode = MODE_SLEW_HISPEED;
     if(fabs(speed) < 128.0) {
@@ -1710,7 +1804,15 @@ void ahp_gt_start_tracking_thread() {
         return;
     devices[ahp_gt_get_current_device()].threads_running = 1;
     pthread_create(&devices[ahp_gt_get_current_device()].tracking_thread, NULL, (void*)&devices[ahp_gt_get_current_device()], track);
+}
 
+void ahp_gt_stop_tracking_thread() {
+    if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
+        return;
+    if(!devices[ahp_gt_get_current_device()].threads_running)
+        return;
+    devices[ahp_gt_get_current_device()].threads_running = 0;
+    pthread_join(devices[ahp_gt_get_current_device()].tracking_thread, NULL);
 }
 
 void ahp_gt_set_tracking_mode(int mode) {
@@ -1719,6 +1821,12 @@ void ahp_gt_set_tracking_mode(int mode) {
     devices[ahp_gt_get_current_device()].tracking_mode = mode;
 }
 
+int ahp_gt_get_tracking_mode() {
+    if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
+        return 0;
+    return devices[ahp_gt_get_current_device()].tracking_mode;
+}
+
 void ahp_gt_start_tracking(int axis) {
     if(!ahp_gt_is_detected(ahp_gt_get_current_device()))
         return;
